import Globals
import os, sys
import re
import time
import datetime

from rpcmethods import *

''' 
    -   parse string actions and create function 
            which can be called later
    -   still simple now, no escaping ... reserved characters are ; () , {}

    -   Examples:  rpc.lightsKeepOn(1,3); door1.Open()  
                   heating1.setWindowOpen({0})
                   httpsrv.get('/xxx/bbb/{1}/{0}')

    -   first word of action must be device name defined in config
    -   {0} - value of action - usually 0/1 or analog value
    -   {1} - calling object name
    -   {2} - calling object circuit (number)
'''

def rpc_action_generator(device, method, args):

    def call(value =''):
        try:
            fun = getattr(device.rpcmethods, method)
            nargs = (a.format(value) for a in args)
            res = fun(*nargs)
            if asyncio.iscoroutine(res): asyncio.create_task(res)
        except:
            Globals.logger.critical("Error exec action %s: %s %s", method,sys.exc_info()[0],sys.exc_info()[1])

    return call


def action_generator(device, method, args):
    if not hasattr(device, method): return None
    fun = getattr(device, method)

    def call(value =''):
        try:
            nargs = (a.format(value) for a in args)
            res = fun(*nargs)
            if asyncio.iscoroutine(res): asyncio.create_task(res)
        except:
            Globals.logger.critical("Error exec action %s: %s %s", method,sys.exc_info()[0],sys.exc_info()[1])

    return call


def multi_action_generator(functions):
    def call(value =''):
        for fun in functions: fun(value)
    return call


def parse_actions(actions, name='', circuit=''):

    if type(actions) is str: actions = actions.split(';')
    functions = []
    for action in actions:
        try:
            devname, method = action.strip().split('.',1)
        except:
            Globals.logger.critical('Action must be in form device.method %s', str(action))
            continue
        device = Units[devname]
        if not device: 
            Globals.logger.critical('Device %s in action undefined %s', devname, str(action))
            continue
        m = re.search('^([^\(]+)\(([^\)]*)\)$',method)
        if m:
            method = m.group(1).strip()
            args = m.group(2).strip()
            args = args.split(',') if args != '' else []
            # test arguments 
            try:
                args = list(a.format("{0}",name,circuit) for a in args)
            except:
                logger.critical("Bad argument format in action %s: %s %s", action, sys.exc_info()[0],sys.exc_info()[1])
                continue

            if hasattr(device, 'rpcmethods'):
                fun = rpc_action_generator(device, method, args)
            else:
                fun = action_generator(device, method, args)
            if fun: functions.append(fun)

    if len(functions) == 0: return None
    if len(functions) == 1: return functions[0]
    return multi_action_generator(functions)
