import os, sys
import six

from celery import Celery
try:
    from Queue import Queue
except (ImportError, AttributeError):
    from queue import Queue

import threading
import time, datetime, pytz

import Globals
from rpcmethods import *

@six.add_metaclass(DevMeta)
class EvCelery(threading.Thread):

    __devname__ = 'brokers'

    def __init__(self, name, rpi_id, remote_server, cmd_prefix):
        threading.Thread.__init__(self)
        self.name = name
        self.circuit = parse_name(name)
        self.rpi_id = rpi_id
        self.cmd_prefix = cmd_prefix
        self.remote_server = remote_server
        self.queue = Queue(maxsize=50)
        self.timezone = pytz.timezone('utc')


    def run(self):
        while True:
            try:
                celery = Celery(str(self.name), broker=str(self.remote_server))
                Globals.logger.info('Celery initialized to %s at %s', str(self.name), str(self.remote_server))
                #celery.conf.task_default_queue = 'celery'
                #celery.conf.task_default_exchange = 'celery'
                celery.conf.task_protocol = 1
                while True:
                    method, items = self.queue.get(self, timeout=None)
                    #method = items[0]
                    #obj = items[1]
                    #nargs = [str(self.rpi_id), str(obj.circuit)]
                    #nargs += items[2:]
                    method = "%s.%s"%(self.cmd_prefix, method)
                    #print(items)
                    celery.send_task(method, args=items, argw={})
                    #Celery.send_task(method, *items, argw={})
                    Globals.logger.debug('%s(%s)', method, str(items))

            except KeyboardInterrupt:
                break
            except Exception as E:
                Globals.logger.critical('Celery failed %s at %s', sys.exc_info()[0],sys.exc_info()[1])
            time.sleep(30)

    def Send(self, method, obj, args):
        try:
            if obj:
                dt = datetime.datetime.now(tz=self.timezone)
                #nargs = [str(self.rpi_id), str(obj.circuit), dt.isoformat()] + [ str(arg) for arg in args ] ##list(args)
                nargs = [str(self.rpi_id), str(obj.circuit), dt.isoformat()] + list(args)
                #print(nargs)
            else:
                nargs = [str(self.rpi_id),] + list(args)
            self.queue.put_nowait((method, nargs))
        except:
            Globals.logger.critical('Celery queue put failed %s at %s', sys.exc_info()[0],sys.exc_info()[1])

Foptions = {
    'rpi_id':        {'type': str, 'help': 'Controller id' },
    'remote_server': {'type': str, 'help': 'Url to connect to. Usually amqp://guest@81.19.0.22/' },
    'cmd_prefix':    {'type': str, 'help': 'Prefix prepended to sending commands. Default remote', 'default': 'remote' },
}


def YFactory(Node, devname, parent=None):
    kwargs = get_kwargs(Foptions, Node)
    return EvCelery(devname, **kwargs)
