import Globals
import six
import asyncio

from rpcmethods import *
import server
import broker
import actions
from  . import virtualgpio as GPIO

@six.add_metaclass(DevMeta)
class NFC:
    __devname__ = "nfcreaders"
    C_REFRESH = 300

    def __init__(self, name, channel, cards=[], actions=None):
        self.name = name
        self.circuit = parse_name(name)
        self.reader = channel
        self.keys = cards #valid cards
        self.action = actions
        self.retries = 0

    def check(self):
        device = check_dev_type(self.reader, GPIO.StringGenerator, 'Nfc %s'%self.name, 'channel')
        device.add_callback(self.tagDetected)
        self.reader = device
        if self.action:
            self.action = actions.parse_actions(self.action, self.name, self.circuit)

    async def tagDetected(self, tag):
        broker.callobj('cardEvent', self, tag)
        Globals.logger.info('Found nfc tag:%s', tag)
        if tag in self.keys and self.action:
            self.action(tag)

    def start(self):
        async def create_task(app):
            asyncio.create_task(self.run())
        server.register_background_task(create_task)

    async def run(self):
        while True:
            try:
                broker.callobj('cardRefresh', self)
            except BaseException as E:
                Globals.logger.critical('Broker call failed: %s at %s', sys.exc_info()[0],sys.exc_info()[1])
            try:
                await asyncio.sleep(self.C_REFRESH)
            except asyncio.CancelledError as E:
                break

    def setupTags(self, keys):
        keys = keys.replace(" ", "")
        keys = keys.split(',')
        self.keys = keys


Foptions = {
    'channel': {'type': 'node', 'help': 'Link to tag generator' },
    'actions': {'default':[], 'type': list, 'help': 'actions to call on valid tag, default empty',
                              'conv': split_by_semicolon},
    'cards':   {'default':[], 'type': list, 'help': 'allowed card tags, default empty', 
                              'conv': split_by_any}
}

def YFactory(Node, devname, parent=None):
    kwargs = get_kwargs(Foptions, Node)
    return NFC(devname, **kwargs)

'''
  register all remote procedures
'''
def Methods():
    nfcs = NFC.__devlist__
    setattr(RPCMethods,'nfcSetupTags',rpc_circuit_one(nfcs,'setupTags'))


# register rpc on using this module
Methods()
