import Globals
from . import virtualgpio as GPIO
import time
import datetime
import six

from rpcmethods import *
import broker

@six.add_metaclass(DevMeta)
class Door:

    __devname__  = 'doors'

    def __init__(self, name, open_pin, close_sensor, is_NO, pulse):
        self.name = name
        self.circuit = parse_name(name)
        self.open_pin = open_pin
        self.close_sensor = close_sensor
        self.is_NO = is_NO
        self.is_locked = True
        self.pulse = pulse

    def check(self):
        if self.open_pin:
            device = check_dev_type(self.open_pin, GPIO.gpout, 'Door %s'%self.name, 'open_pin')
            self.open_pin = device

        if (self.close_sensor):
            device = check_dev_type(self.close_sensor, GPIO.gpin, 'Door %s'%self.name, 'close_sensor')
            self.close_sensor = device
            if device:
                if self.is_NO: device.set_reversed()
                device.add_event_detect(GPIO.BOTH, callback=self.doorEvent)
                device.add_event_start(callback=self.doorEvent)

        Globals.logger.debug("Door %s initialized",self.name)


    def sendUpdate(self):
        broker.callobj('doorEvent',self,self.is_opened, self.is_locked)
        return "OK"

    def getState(self):
        return self.is_opened

    async def lock(self):
        if (not self.open_pin):
            return "This door has no electric lock"
        await self.open_pin.output(GPIO.R_OFF)
        self.is_locked = True
        Globals.logger.debug("Door %s locked", self.name)
        return "OK"

    async def unlock(self):
        if (not self.open_pin):
            return "This door has no electric lock"
        await self.open_pin.output(GPIO.R_ON)
        self.is_locked = False
        Globals.logger.debug("Door %s unlocked", self.name)
        return "OK"

    async def doorEvent(self, gpin):
        #for Normally Openned door sensor-- Normally Openned when door are closed
        self.is_opened = gpin.value != 1
        if (self.is_opened):
            Globals.logger.debug("Door %s opened", self.name)
            if self.is_locked:
                await self.open_pin.output(GPIO.R_OFF)
        else:
            Globals.logger.debug("Door %s closed", self.name)
        self.sendUpdate()

    async def open(self):
        if (not self.open_pin):
            return "This door has no electric lock"
        if (self.is_locked):
            await self.open_pin.set_for_time(GPIO.R_ON, self.pulse)
            Globals.logger.debug("Door %s unlocked for %3.1fs", self.name, self.pulse)
        return "OK"

Foptions = {
    'open_pin':       {'type': 'node', 'help': 'Link to some relay. Can be undefined', 'default': None },
    'close_sensor':   {'type': 'node', 'help': 'Link to some input. Can be undefined', 'default': None  },
    'is_NO'       :   {'type': int,    'help': 'Reversed sense of sensor 1=door opened', 'default': 0 },
    'pulse'       :   {'type': float,    'help': 'Length of pulse for openning [seconds]', 'default': 3.0 },
}

def YFactory(Node, devname, parent=None):
    kwargs = get_kwargs(Foptions, Node)
    return Door(devname, **kwargs) #open_pin, close_sensor, is_NO)

'''
  register all remote procedures
'''
def Methods():
    doors = Door.__devlist__
    setattr(RPCMethods,'doorGetState',rpc_circuit(doors,'getState'))
    setattr(RPCMethods,'doorSendUpdate',rpc_circuit(doors,'sendUpdate'))
    setattr(RPCMethods,'doorOpen',rpc_circuit(doors,'open'))
    setattr(RPCMethods,'doorLock',rpc_circuit(doors,'lock'))
    setattr(RPCMethods,'doorUnlock',rpc_circuit(doors,'unlock'))


# register rpc on using this module
Methods()
