import Globals
from . import virtualgpio as GPIO
import time
import datetime
import six

from rpcmethods import *
import broker

@six.add_metaclass(DevMeta)
class Fan:

    __devname__  = 'fans'

    def __init__(self, name, pin):
        self.name = name
        self.circuit = parse_name(name)
        self.pin = pin
        self.fan_status= False

    def check(self):
        if self.pin:
            device = check_dev_type(self.pin, GPIO.gpout, 'Fan %s'%self.name, 'pin')
            device.add_callback(callback=self.relayEvent)
            self.pin = device
            Globals.logger.debug("Fan %s initialized",self.name)

    def sendUpdate(self):
        broker.callobj('fanEvent',self, self.fan_status)
        return "OK"

    def getState(self):
        return self.fan_status

    async def relayEvent(self, value):
        ''' acknoledgment of relay status - called from gpout'''
        self.fan_status = value
        broker.callobj('fanEvent',self, self.fan_status)

    async def setState(self, value):
        ''' action: to switch on/off the fan ''' 
        value = value in (True, 1, '1', 'True')
        await self.pin.output(GPIO.R_ON if value else GPIO.R_OFF)
        Globals.logger.debug("Fan %s turned %s", self.name, "on" if value else 'off')
        return "OK"

    async def runForTime(self, time):
        ''' action: switch the fan on for time interval (in seconds)''' 
        await self.pin.set_for_time(GPIO.R_ON, time)
        Globals.logger.debug("Fan %s turned on for %s secs", self.name, time)

Foptions = {
    'pin':       {'type': 'node', 'help': 'Link to some relay' }
}

def YFactory(Node, devname, parent=None):
    kwargs = get_kwargs(Foptions, Node)
    return Fan(devname, **kwargs)


def Methods():
    fans = Fan.__devlist__
    setattr(RPCMethods,'fanGetState',rpc_circuit(fans,'getState'))
    setattr(RPCMethods,'fanSendUpdate',rpc_circuit(fans,'sendUpdate'))
    setattr(RPCMethods,'fanSet',rpc_circuit_one(fans,'setState'))
    setattr(RPCMethods,'fanKeepOn',rpc_circuit_one(fans,'runForTime'))

Methods()

  
