import Globals
from . import virtualgpio as GPIO
import time
import datetime
import six
import asyncio

from rpcmethods import *
import broker

@six.add_metaclass(DevMeta)
class Light:

    __devname__  = 'lights'

    def __init__(self, name, switch_pin, light_pin):
        self.name = name
        self.circuit = parse_name(name)
        self.switch_pin = switch_pin
        self.light_pin = light_pin
        self.light_status= False

    def check(self):
        if self.light_pin:
            device = check_dev_type(self.light_pin, GPIO.gpout, 'Light %s'%self.name, 'light_pin')
            device.add_callback(callback=self.relayEvent)
            self.light_pin = device

        for pin in self.switch_pin:
            device = check_dev_type(pin, GPIO.gpin, 'Light %s'%self.name, 'switch_pin')
            #self.switch_pin = device
            if device:
                device.add_event_detect(GPIO.RISING, callback=self.lightEvent)

    def sendUpdate(self):
        broker.callobj('lightEvent',self, self.light_status)
        return "OK"

    def getState(self):
        return self.light_status

    async def relayEvent(self, value):
        ''' acknoledgment of relay status - called from gpout'''
        self.light_status = value
        broker.callobj('lightEvent',self, self.light_status)

    async def lightEvent(self, gpin):
        ''' incoming event: called from gpin (wall switch)'''
        await self.setState(not self.light_status)

    async def setState(self, value):
        ''' action: to switch on/off the light ''' 
        #print('setstate', self.name, value)
        await self.light_pin.output(GPIO.R_ON if value in (True, 1, '1', 'True') else GPIO.R_OFF)
        Globals.logger.debug("Light %s turned %s", self.name, "on" if value else 'off')
        return "OK"

    async def lightForTime(self, time):
        ''' action: switch the light on for time interval (in seconds)''' 
        await self.light_pin.set_for_time(GPIO.R_ON, time)
        Globals.logger.debug("Light %s turned on for %s secs", self.name, time)

Foptions = {
#    'switch_pin':   {'type': 'node', 'help': 'Link to some input' },
    'switch_pin':   {'default':[], 'type': list, 'help': 'Link to input[s]', 
                              'conv': split_by_any},
    'light_pin':     {'type': 'node', 'help': 'Link to some relay' },
}

def YFactory(Node, devname, parent=None):
    kwargs = get_kwargs(Foptions, Node)
    return Light(devname, **kwargs)


def Methods():
    lights = Light.__devlist__
    setattr(RPCMethods,'lightGetState',rpc_circuit(lights,'getState'))
    setattr(RPCMethods,'lightSendUpdate',rpc_circuit(lights,'sendUpdate'))
    "		            	switch light on (1)/off (0)"
    setattr(RPCMethods,'lightSet',rpc_circuit_one(lights,'setState'))
    "            			turn on the light if its off and keep it on for secs"
    setattr(RPCMethods,'lightKeepOn',rpc_circuit_one(lights,'lightForTime'))
    setattr(RPCMethods,'lightSetAll',rpc_circuit(lights,'setState'))

Methods()
