import Globals
from . import virtualgpio as GPIO
import six

from rpcmethods import *
import broker
import actions

@six.add_metaclass(DevMeta)
class Window:

    def __init__(self, name, pin, actions):
        self.name = name
        self.circuit = parse_name(name)
        self.pin = pin
        self.action = actions


    def check(self):

        device = check_dev_type(self.pin, GPIO.gpin, 'Window %s'%self.name, 'pin')
        self.pin = device
        if device:
            device.add_event_detect(GPIO.BOTH, callback=self.windowEvent)
            device.add_event_start(callback=self.windowEvent)
        if self.action:
            self.action = actions.parse_actions(self.action, self.name, self.circuit)
        #if self.heating_circuit:
        #  device = check_dev_type(self.heating_circuit, Heating, 'Window %s'%self.name, 'heating_circuit')
        #  self.heating_circuit = device
        Globals.logger.debug("Window %s initialized",self.circuit)


    def sendUpdate(self):
        broker.callobj('windowEvent',self, self.pin.value)
        return "OK"

    def getState(self):
        return self.pin.value

    async def windowEvent(self, gpin):
        self.sendUpdate()
        Globals.logger.debug("Window %s %s", self.name, "opened" if self.pin.value == 0 else "closed")

        if self.action:
            self.action(self.pin.value)

        #if (self.heating_circuit):
            #self.heating_circuit.setWindowOpened(self.pin.value == 0)
            #action = "heatingSetWindowOpened('%s', %s)" %(self.heating_circuit, self.pin.value)
            #Globals.JsonTo(str("http://localhost:"+str(Globals.local_port)+"/"), str(action))


Foptions = {
    'pin':     {'type': 'node', 'help': 'Link to some input' },
    'actions': {'default':[], 'type': list, 'help': 'actions to call on change state, default empty',
                              'conv': split_by_semicolon},
}


def YFactory(Node, devname, parent=None):
    kwargs = get_kwargs(Foptions, Node)
    return Window(devname, **kwargs)
