#!/usr/bin/python3

from os import listdir
from os.path import isfile, isdir, join
import re
from future.utils import iteritems

from rpcmethods import *
import rpcmethods

class EmptyClass(object):
    pass


def analyze_module(prefix, f):
    devicetype = re.sub('.py$','',f)

    rpcmethods.RPCMethods = EmptyClass()
    try:
        module = __import__("%s.%s" % (prefix,devicetype), fromlist=[prefix,])
    except ImportError as E:
        print(str(E))
        return

    opts = []
    if hasattr(module,'YFactory') and hasattr(module,'Foptions'):
        for name, option in iteritems(module.Foptions):
            sign = '-' if 'default' in option else '*'
            hlp  = option.get('help', '')
            typ  = option.get('type', str)
            if typ != 'node' : typ = typ.__name__
            opts.append(" %c %-15s: %s, [type=%s]" % (sign, name, hlp, typ))

    methods = []
    for name, fun in iteritems(rpcmethods.RPCMethods.__dict__):
        par = ''
        if fun.__name__ == 'circuit_zero' : par = 'circuit'
        elif fun.__name__ == 'circuit_one' : par = 'circuit, value'
        methods.append("%s(%s)" % (name, par))

    return ["%s.%s:" % (prefix, devicetype), opts, methods]


def directory(path, prefix):
    pyfiles = [f for f in listdir(path) if isfile(join(path, f)) \
                          and f.endswith('.py') \
                          and not f.startswith('_') \
                          and not f.startswith('.')]
    #for f in pyfiles: 
    #    analyze_module(prefix, f)

    return [analyze_module(prefix, f) for f in pyfiles]


def main(path):
    dirs = [d for d in listdir(path) if isdir(join(path, d)) and not d.startswith('.') ]
    #onlyfiles = [f for f in listdir(mypath) if isfile(join(mypath, f))]
    doc = { prefix: directory(join(path, prefix), prefix) for prefix in dirs }
    
    print('=================== Node options ========================')
    for prefix, subdoc in iteritems(doc):
        for f in subdoc:
            if f and f[1]:
                print('')
                print(f[0])
                for o in f[1]: print(o)
    print('\n=================== RPC ========================')

    for prefix, subdoc in iteritems(doc):
        for f in subdoc:
            if f and f[2]:
                print('')
                print(f[0])
                for o in f[2]: print("  "+o)
    print('')
    #print(doc)

main('.')