import Globals
import time
import datetime
import sys
import threading
import six

from rpcmethods import *
from .readgroup import ModbusReadGroup
from device import virtualgpio as GPIO

@six.add_metaclass(DevMeta)
class ModbusInput(GPIO.gpin):

    #__devname__  = 'inputs'

    def __init__(self, name, channel, index):
        self.name = name
        self.circuit = parse_name(name)
        self.channel = channel
        self.index = index
        self.value = None
        self.reverse = False
        self.rise_callback = []
        self.fall_callback = []
        self.start_callback = []
        self.set_reversed(False)

    def check(self):
        device = check_dev_type(self.channel, ModbusReadGroup, 'Input %s'%self.name, 'channel')
        self.channel = device
        #self.set_countermode(self.countermode)
        self.channel.register_callback(self.index, self.event0)

    def set_reversed(self, reverse=True):
        '''  values from pigpio/rpi/unipi1 are default REVERSED
        ''' 
        self.reverse = reverse
        if not reverse:
            self.to_value = lambda x: 0 if x==0 else 1
        else:
            self.to_value = lambda x: 1 if x==0 else 0

    async def event0(self, level):
        self.value = self.to_value(level)
        #print("STA: ", self.name, "%08x"%level)
        self.channel.register_callback(self.index, self.event)
        for fun in self.start_callback:
                await fun(self)

    async def event(self, level):
        value = self.to_value(level)
        #print('EVT: ', self.name, "%08x"%level)
        if self.value != value:
            self.value = value
            if value == 1:
                for fun in self.rise_callback:
                    await fun(self)
            elif value == 0:
                for fun in self.fall_callback:
                    await fun(self)

    def add_event_start(self, callback):
        self.start_callback.append(callback)

    def add_event_detect(self, event, callback):
        if event == GPIO.BOTH: 
            self.rise_callback.append(callback)
            self.fall_callback.append(callback)
        elif event == GPIO.RISING:
            self.rise_callback.append(callback)
        elif event == GPIO.FALLING:
            self.fall_callback.append(callback)


Foptions = {
    'channel':  {'type': 'node', 'help': 'Link to Modbus readgroup' },
    'index':   {'type': int, 'help': 'Index to readgroup' },
    #'debounce': {'type': int, 'help': 'Debouncing time in ms. default not set', 'default': None, 'conv': None_or_int},
    #'countermode': {'type': str, 'help': 'How to count pulses. Allowed disabled,rising, falling. Default disabled' ,'default': 'disabled'},
}

def YFactory(Node, devname, parent = None):
    kwargs = get_kwargs(Foptions, Node, parent)
    return ModbusInput(devname, **kwargs)


