import Globals
import time
import datetime
import sys
import threading
import six

from rpcmethods import *
from .client import BaseModbusClient
from device import virtualgpio as GPIO

@six.add_metaclass(DevMeta)
class ModbusRelay(GPIO.gpout):

    def __init__(self, name, channel, unit, coil, startup_value):
        self.name = name
        self.circuit = parse_name(name)
        self.mbclient = channel
        self.unit = unit
        self.coil = coil
        self.startup_value = startup_value
        self.value = None
        self.callback = []
        self.timer = None
        self.running_lock = asyncio.Lock()

    def check(self):
        device = check_dev_type(self.mbclient, BaseModbusClient, 'Output %s'%self.name, 'channel')
        self.mbclient = device
        #self.mbclient.registerDevice(self.coil, self.event)

    async def output(self, value):
        async with self.running_lock:
            timer = self.timer
            self.timer=None
            if timer: timer.cancel()
            await self.mbclient.write_coil(self.coil, 1 if value==GPIO.R_ON else 0, self.unit)

    async def timer_callback(self, value, timeout):
        await asyncio.sleep(timeout)
        async with self.running_lock:
            self.timer=None
            await self.mbclient.write_coil(self.coil, 1 if value==GPIO.R_ON else 0, self.unit)


    async def set_for_time(self, value, timeout):
        async with self.running_lock:
            timer = self.timer
            self.timer=None
            if timer: timer.cancel()
            await self.mbclient.write_coil(self.coil, 1 if value==GPIO.R_ON else 0, self.unit)
            self.timer = asyncio.create_task(self.timer_callback(0 if value==GPIO.R_ON else 1, timeout))

    async def event0(self, item):
        self.chip.register_relay(self._mask, self.event)
        self.value = GPIO.R_OFF if item==0 else GPIO.R_ON

        await asyncio.gather(*(x(self.value) for x in self.callback))
        if not(self.startup_value is None):
            await self.output(self.startup_value)

    async def event(self, item):
        value = GPIO.R_OFF if item==0 else GPIO.R_ON
        self.value = value
        await asyncio.gather(*(x(self.value) for x in self.callback))

    def add_callback(self, callback):
        self.callback.append(callback)



Foptions = {
    'channel':{'type': 'node', 'help': 'Link to modbus client' },
    'unit':   {'type': int, 'help': 'Unit modbus address. Default 0', 'default': 0 },
    'coil':   {'type': int,  'help': 'Coil address in unit modbus map' },
    'startup_value':   {'type': int,  'help': 'Value to set during boot. Default do not set', 'default': None, 'conv': None_or_int }
}

def YFactory(Node, devname, parent = None):
    kwargs = get_kwargs(Foptions, Node, parent)
    return ModbusRelay(devname, **kwargs)
