import Globals
import sys
import asyncio
import six
import serial_asyncio as serial

from rpcmethods import *
import server

from .anetronix import NetronixProtocol

@six.add_metaclass(DevMeta)
class NfcBus:

    def __init__(self, name, port, baudrate):
        self.name = name
        self.circuit = parse_name(name)
        self.portname = port
        self.baudrate = baudrate
        self.lock = asyncio.Lock()
        self.timeout = 0.02
        self._readers = {}
        self.protocol = None
        self._Xreaders = set(())

    def check(self):
        pass

    def registerDevice(self, reader):
        self._readers[str(reader.address)] = reader

    def start(self):
        async def create_task(app):
            asyncio.create_task(self.run())
        server.register_background_task(create_task)


    async def send(self, address, led):
        if not(led in (2,3)) : return
        async with self.lock:
            await self.protocol.SetIO(address, led, 1)
        return True


    async def run(self):

        startEvent = asyncio.Event()
        class Protocol(NetronixProtocol):

            def connection_made(obj, transport):
                NetronixProtocol.connection_made(obj,transport)
                self.protocol = obj
                startEvent.set()
                #print('port opened', transport.serial)

            def connection_lost(obj, exc):
                NetronixProtocol.connection_lost(obj, exc)
                print('port closed')

        loop = asyncio.get_event_loop()
        coro = serial.create_serial_connection(loop, Protocol, self.portname, baudrate=self.baudrate)
        asyncio.create_task(coro)
        #loop.run_until_complete(coro)

        await startEvent.wait()

        while True:
            try:
                async with self.lock:
                    responses = await self.protocol.ScanBus()
                    await asyncio.sleep(0.02)
                if responses:
                    for dev, EventQueueStatus in iteritems(responses):
                        #Globals.logger.info("NfcBus %s: %s", self.name, dev)
                        try:
                            reader = self._readers[dev]
                        except KeyError:
                            if not(dev in self._Xreaders):
                                Globals.logger.info("NfcBus %s: Unconfigured reader id=%s", self.name, dev)
                                self._Xreaders.add(dev)
                            continue
                        async with self.lock:
                            events = await self.protocol.ReadNewEvents(EventQueueStatus, reader)
                            [ await reader.Worker(event) for event in events ]

            except asyncio.CancelledError as E:
                break
            except BaseException as E:
                Globals.logger.critical("NfcBus %s: %s %s", self.name, sys.exc_info()[0],sys.exc_info()[1])
            await asyncio.sleep(self.timeout)


Foptions = {
    'port':     {'type': str, 'help': 'Serial port' },
    'baudrate': {'type': int, 'help': 'Serial port baudrate' },
}


def YFactory(Node, devname, parent = None):
    kwargs = get_kwargs(Foptions, Node)
    return NfcBus(devname, **kwargs)
