import Globals
import sys
import six

import json, re
from rpcmethods import *
from .reader import NfcReader
from device import virtualgpio as GPIO

@six.add_metaclass(DevMeta)
class NfcButton(GPIO.gpin):

    #__devname__  = 'inputs'

    def __init__(self, name, channel):
        self.name = name
        self.circuit = parse_name(name)
        self.reader = channel
        self.reverse = False
        self.rise_callback = []
        self.fall_callback = []
        self.set_reversed(False)
        self.value = self.to_value(0)

    def check(self):
        device = check_dev_type(self.reader, NfcReader, 'Input %s'%self.name, 'channel')
        self.reader = device
        self.reader.registerGpin(self.event)

    def set_reversed(self, reverse=True):
        self.reverse = reverse
        if reverse:
            self.to_value = lambda x: 1 if x==0 else 0
        else:
            self.to_value = lambda x: 0 if x==0 else 1
        
    #def sendUpdate(self):
    #    return self.evokws.sendUpdate()

    async def event(self, address, item):
        value = self.to_value(item)
        if self.value == 0 and value == 1:
            #print("Input %s RISE" % self.circuit, value)
            self.value = value
            for fun in self.rise_callback:
                await fun(self)
        elif self.value == 1 and value == 0:
            #print("Input %s FALL" % self.circuit, value)
            self.value = value
            for fun in self.fall_callback:
                await fun(self)
        # !!! generates only rising events
        self.value = self.to_value(0)
        #self.value = value

    def add_event_start(self, callback):
        pass

    def add_event_detect(self, event, callback):
        if event == GPIO.BOTH: 
            self.rise_callback.append(callback)
            self.fall_callback.append(callback)
        elif event == GPIO.RISING:
            self.rise_callback.append(callback)
        elif event == GPIO.FALLING:
            self.fall_callback.append(callback)

Foptions = {
    'channel':  {'type': 'node', 'help': 'Link to Netronix bus' },
}

def YFactory(Node, devname, parent = None):
    kwargs = get_kwargs(Foptions, Node, parent)
    return NfcButton(devname, **kwargs)

