import Globals
import six
import asyncio

from rpcmethods import *
import broker
import actions

from .bus import NfcBus
from device import virtualgpio as GPIO

#from .netronix import NetronixDevice

@six.add_metaclass(DevMeta)
class NfcReader(GPIO.StringGenerator):

    def __init__(self, name, channel, address):
        self.name = name
        self.circuit = parse_name(name)
        self.bus = channel
        self.address = address
        self.gpin_callback = None
        self.callbacks = []

    def check(self):
        device = check_dev_type(self.bus, NfcBus, 'Input %s'%self.name, 'channel')
        self.bus = device
        self.bus.registerDevice(self)

    def add_callback(self, fun):
        self.callbacks.append(fun)

    def registerGpin(self, callback):
        self.gpin_callback = callback

    async def sendLed(self, pin):
        return await self.bus.send(self.address, pin)

    def tagDetected(self, tag):
        for fun in self.callbacks:
            res = fun(tag)
            if asyncio.iscoroutine(res): asyncio.create_task(res)

    async def Worker(self, params):
        ''' parsed events receiver '''
        if not params: return
        tag, button = params
        Globals.logger.debug("Nfc %s on Bus %s: tag=%s button=%s", self.name, self.bus.name, tag, button)
        if tag:
            self.tagDetected(tag)
        if button and self.gpin_callback:
            await self.gpin_callback(self.address,button)


Foptions = {
    'channel':       {'type': 'node', 'help': 'Link to Netronix bus' },
    'address':       {'type': int, 'help': 'Addres of reader on bus' },
}

def YFactory(Node, devname, parent = None):
    kwargs = get_kwargs(Foptions, Node, parent)
    return NfcReader(devname, **kwargs)

