import time
import six

from rpcmethods import *
from .owbus import OwBus
from device import virtualgpio as GPIO

@six.add_metaclass(DevMeta)
class OwThermo(GPIO.Thermometer):

    __devname__  = 'thermometers'

    def __init__(self, name, channel, address):
        self.name = name
        self.circuit = parse_name(name)
        self.channel = channel
        self.address = address

    def check(self):
        device = check_dev_type(self.channel, OwBus, 'Thermometer %s'%self.name, 'channel')
        self.channel = device
        device.register_persistent(self.address)

    def get_valid(self, interval):
        temprec = self.channel.get_persistent(self.address)
        stime = asyncio.get_running_loop().time()
        if temprec and (stime < temprec['time'] + interval):
            return  temprec['value']
        return None

Foptions = {
    'channel': {'type': 'node', 'help': 'Link to evok bus' },
    'address':      {'type': str, 'help': 'Bus identification of sensor' }
}


def YFactory(Node, devname, parent = None):
    kwargs = get_kwargs(Foptions, Node, parent)
    return OwThermo(devname, **kwargs)
