import logging
import Globals
import re
from future.utils import iteritems
import asyncio


class DevList:
    """ modified dictionary 
        - return None if index is unknown
        - key can be number - converted to string
        - add - insert device with key
    """

    def __init__(self, name, methods=None):
        self._list = {}
        self.name = name
        self.methods = methods

    def __getitem__(self, key):
        try:
            return self._list[str(key)]
        except KeyError:
            return None

    def __setitem__(self, key, val):
        self._list[str(key)] = val

    def __iter__(self, key, val):
        for i in self._list: yield i

    def iteritems(self):
        for k,v in iteritems(self._list): yield k,v

    def __len__(self):
        return len(self._list)

    def __contains__(self,key):
        return str(key) in self._list

    def add(self, key, value):
        self._list[str(key)] = value

    def addMethods(self):
        if self.methods: self.methods(self)


class DevListFactory:

    list_of_devlist = {}

    def __call__(self, name):
        try:
            return self.list_of_devlist[name]
        except KeyError:
            d = DevList(name)
            self.list_of_devlist[name] = d
            return d

DevListFactory=DevListFactory()
Units = DevListFactory('__all__')

# modify class to:
#    -- create DevList with name of class or __devname__
#    -- during instance __init__ do automatic registration to DevList and Units with key self.circuit

class DevMeta(type):

    def __init__(cls, clsname, parents, props):
        fun = props['__init__'] if '__init__' in props else None
        def init_decorator_devlist(self,*fargs, **kwargs):
            if fun: fun(self,*fargs, **kwargs)
            self.__devlist__.add(self.circuit, self)
            Units.add(self.name, self)

        def init_decorator(self,*fargs, **kwargs):
            if fun: fun(self,*fargs, **kwargs)
            Units.add(self.name, self)

        if '__devname__' in props:
            devname = props['__devname__']
            setattr(cls, '__devlist__', DevListFactory(devname))
            setattr(cls,'__init__',init_decorator_devlist)
        else:
            setattr(cls,'__init__',init_decorator)

    def __getitem__(cls, key):
        return cls.__devlist__[key]



class RPCMethods:
    pass

'''
    def __init__(self, rpi_id):
        self.rpi_id = rpi_id

    def sayHi(self):
        return "Hi-"+str(self.rpi_id)
'''


#method generators
def rpc_circuit(devices,func):
    async def circuit_zero(circuit):
        dev = devices[circuit]
        if not dev: return "Unconfigured device %s%s" % (devices.name,circuit)
        f = getattr(dev,func)
        if not f: return "Undefined function"
        res = f()
        if asyncio.iscoroutine(res): return await res
        return res

    return circuit_zero

def rpc_circuit_one(devices,func):
    async def circuit_one(circuit, par1):
        dev = devices[circuit]
        if not dev: return "Unconfigured device %s%s" % (devices.name,circuit)
        f = getattr(dev,func)
        if not f: return "Undefined function"
        res = f(par1)
        if asyncio.iscoroutine(res): return await res
        return res

    return circuit_one

def rpc_all_one(devices,func):
    async def all_one(par1):
        for dev in devices:
            f = getattr(dev,func)
            if f: f(par1)
            res = f()
            if asyncio.iscoroutine(res): await res
        return "OK"

    return all_one



def check_dev_type(devname, reqclass, unitname, parameter):
    if isinstance(devname,reqclass):
        return devname

    device = Units[devname]
    if not device:
        Globals.logger.critical("%s has unknown %s = %s", unitname, parameter, devname)
    elif not isinstance(device,reqclass):
        Globals.logger.critical("%s - %s = %s is not an %s", unitname, parameter, devname, reqclass.__name__)
    else:
        return device
    return None


def try_to_start(unit):

    if hasattr(unit, 'start') and not(hasattr(unit, 'started') and unit.started):
        unit.start()
        unit.started = True


def parse_name(name):
    res = re.search('^(.*\D)(\d+)$', name)
    if res: return res.group(2)
    return None


def None_or_int(s):
    return s if s is None else int(s)

def split_by_semicolon(s):
    return [ k.strip() for k in s.split(';')]

def split_by_any(s):
    return [ k.strip() for k in re.split('\s*[\s,;]\s*',s)]

def get_kwargs(Foptions, Node, channel=None):
    kwargs = {}
    for name, option in iteritems(Foptions):
        if 'default' in option:
            val = Node.get(name, option['default'])
        else:
            if channel and name == 'channel' : 
                val = Node.get(name, channel)
            else:
                val = Node[name]

        if 'type' in option:
            mytype = option['type']
            if mytype != 'node' and (type(val) != mytype):
                try:
                    val =  option['conv'](val)
                except KeyError:
                    val = mytype(val)

        kwargs[name] = val
    return kwargs
